/*
	File:			host.h

	Description:	Host header file.

	Author:			Roger Flores

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/

/* Define how some common types are defined on the
host machine.
*/

typedef signed char int8;
typedef unsigned char uint8;
typedef signed short int16;
typedef unsigned short uint16;
typedef long int32;
typedef unsigned long uint32;
typedef long intptr;
typedef unsigned long uintptr;

#ifndef __cplusplus
typedef unsigned char bool;
#endif

#ifndef NULL
#define NULL	0
#endif

#include <TapWave.h>
#include "t3dlib1_AK.h"

// The API to the host functions is C based.
#ifdef __cplusplus
extern "C"
{
#endif


#define FRICTION_FACTOR  (float)(0.05)  // friction of the virtual space

// these are the gravity constants, they are selected to simply work
#define VIRTUAL_GRAVITY_CONSTANT     (float)0.01
#define SHIP_MASS                    (float)2
#define BLACK_HOLE_MASS              (float)50000
#define MAX_VEL                      (float)30

#define rotateLeftKey		keyBitHard1		// polled every game period
#define rotateRightKey		keyBitHard2		// polled every game period
#define shootKey				keyBitHard3		// polled every game period
#define thrustKey				keyBitHard4		// polled every game period
#define warpKey				keyBitPageUp	// polled every game period
//#define keysAllowedMask		(rotateLeftKey | rotateRightKey | shootKey | thrustKey | warpKey)
#define restartGameChar		pageDownChr
#define rotateLeftKeyRepeatDelay		2		// polled every game period
#define rotateRightKeyRepeatDelay	2		// polled every game period
#define shootKeyRepeatDelay			4		// polled every game period
#define thrustKeyRepeatDelay			2		// polled every game period


// defines for particle system
#define PARTICLE_STATE_DEAD               0
#define PARTICLE_STATE_ALIVE              1

// types of particles
#define PARTICLE_TYPE_FLICKER             0
#define PARTICLE_TYPE_FADE                1 

// color of particle
#define PARTICLE_COLOR_RED                0
#define PARTICLE_COLOR_GREEN              1
#define PARTICLE_COLOR_BLUE               2
#define PARTICLE_COLOR_WHITE              3

#define MAX_PARTICLES                     400

// color ranges
#define COLOR_RED_START                   0
#define COLOR_RED_END                     5

#define COLOR_GREEN_START                 12
#define COLOR_GREEN_END                   17

#define COLOR_BLUE_START                  6
#define COLOR_BLUE_END                    11

#define COLOR_WHITE_START                 18
#define COLOR_WHITE_END                   23

// size of universe
#define UNIVERSE_MIN_X   (-8000)
#define UNIVERSE_MAX_X   (8000)
#define UNIVERSE_MIN_Y   (-8000)
#define UNIVERSE_MAX_Y   (8000)

// weapons defines
#define MAX_PLASMA         32   // max number of plasma pulses
#define PLASMA_SPEED       16   // pixels/sec of plasma pulse
#define PLASMA_SPEED_SLOW  12   // slow version for gunships

#define PLASMA_STATE_OFF   0   // this plasma is dead or off
#define PLASMA_STATE_ON    1   // this one is alive and in flight
#define PLASMA_RANGE_1     30  // type 1 plasma disruptor

#define PLASMA_ANIM_PLAYER     0  // a plasma torpedeo from player
#define PLASMA_ANIM_ENEMY      1  // a plasma torpedeo from any enemy

// asteroid field defines
#define MAX_ROCKS             300
#define ROCK_STATE_OFF        0   // this rock is dead or off
#define ROCK_STATE_ON         1   // this one is alive and in flight

#define ROCK_LARGE            0   // sizes of rock
#define ROCK_MEDIUM           1
#define ROCK_SMALL            2

// explosion defines 
#define MAX_BURSTS            8
#define BURST_STATE_OFF       0   // this burst is dead or off
#define BURST_STATE_ON        1   // this one is alive

// defines for player states
#define PLAYER_STATE_DEAD           0
#define PLAYER_STATE_DYING          1
#define PLAYER_STATE_INVINCIBLE     2
#define PLAYER_STATE_ALIVE          3
#define PLAYER_REGEN_COUNT          100 // click before regenerating
#define AMMO_REGEN_COUNT			250
#define WRAITH_INDEX_DIR            3 // index of direction var

#define MAX_PLAYER_SPEED            16

// sound id's
#define MAX_FIRE_SOUNDS       8
#define MAX_EXPL_SOUNDS       8

// defines for gunships
#define MAX_GUNSHIPS          8

#define GUNSHIP_RANGE_RING       1000 // distance from player gunship will be created at

#define GUNSHIP_RIGHT            0
#define GUNSHIP_LEFT             1

#define MAX_GUNSHIP_DAMAGE       150
#define INDEX_GUNSHIP_DAMAGE     2  // damage level of gunship
#define INDEX_GUNSHIP_DIR        3  // direction of ship 
#define INDEX_GUNSHIP_TURRET     4  // direction of turret

// life state for gunships
#define GUNSHIP_STATE_DEAD        0
#define GUNSHIP_STATE_ALIVE       1
#define GUNSHIP_STATE_DAMAGED     2  
#define GUNSHIP_STATE_DYING       3
#define GUNSHIP_MIN_ATTACK_RANGE  500

// defines for powerups
#define POWERUP_TYPE_AMMO       0
#define POWERUP_TYPE_SHLD       1

#define POWERUP_STATE_DEAD      0
#define POWERUP_STATE_ALIVE     1
#define MAX_POWERUPS            16
#define INDEX_POWERUP_TYPE      0


// defines for stations
#define MAX_STATIONS           20

// life state for stations
#define STATION_STATE_DEAD       0
#define STATION_STATE_ALIVE      1
#define STATION_STATE_DAMAGED    2  
#define STATION_STATE_DYING      3

#define STATION_SHIELDS_ANIM_ON  0  // animations for shields
#define STATION_SHIELDS_ANIM_OFF 1 

#define STATION_RANGE_RING       300

#define INDEX_STATION_DAMAGE     2  // tracks the current damage of station
#define MAX_STATION_DAMAGE      100

#define STATION_MIN_UPLINK_DISTANCE  150


// defines for mines
#define MAX_MINES                16

// life state for stations
#define MINE_STATE_DEAD       0
#define MINE_STATE_ALIVE      1
#define MINE_STATE_DAMAGED    2  
#define MINE_STATE_DYING      3

#define MINE_STATE_AI_ACTIVATED   1
#define MINE_STATE_AI_SLEEP       0

#define INDEX_MINE_AI_STATE       1  // state of ai system
#define INDEX_MINE_DAMAGE         2  // tracks the current damage of MINE
#define INDEX_MINE_CONTACT_COUNT  3  // tracks how long mine has been in contact with player

#define MAX_MINE_DAMAGE            50 
#define MAX_MINE_CONTACT_COUNT     20

#define MAX_MINE_VELOCITY        16  
#define MIN_MINE_TRACKING_DIST   1000
#define MIN_MINE_ACTIVATION_DIST 250 

// defines for the star field

#define MAX_STARS                256   // number of stars in universe

#define STAR_PLANE_0             0    // far plane
#define STAR_PLANE_1             1    // near plane
#define STAR_PLANE_2             2

#define STAR_COLOR_0             8    // color of farthest star plane
#define STAR_COLOR_1             7
#define STAR_COLOR_2             15   // color of nearest star plane
// indices used to access data arrays in BOBs 
#define INDEX_WORLD_X                     8
#define INDEX_WORLD_Y                     9

// defines for the states of the main loop
#define GAME_STATE_INIT                  0
#define GAME_STATE_MENU                  1
#define GAME_STATE_RESTART               2
#define GAME_STATE_RUNNING               3
#define GAME_STATE_UPLINK                4
#define GAME_STATE_EXIT                  5
#define GAME_STATE_WAITING_FOR_EXIT      6
#define GAME_STATE_PAUSED                7

// the menu defines
#define MENU_STATE_MAIN                  0   // main menu state 
#define MENU_STATE_INST                  1   // instructions state
#define MENU_SEL_NEW_GAME                0
#define MENU_SEL_INSTRUCTIONS            1
#define MENU_SEL_EXIT                    2
#define MAX_INSTRUCTION_PAGES            6  

// defines for font
#define FONT_NUM_CHARS         96  // entire character set
#define FONT_WIDTH             12
#define FONT_HEIGHT            12
#define FONT_WIDTH_NEXT_NUM    8
#define FONT_WIDTH_NEXT_LOWER  7
#define FONT_WIDTH_NEXT_UPPER  8

// number of starting objects
#define NUM_ACTIVE_GUNSHIPS          8
#define NUM_ACTIVE_MINES             8
#define NUM_ACTIVE_STATIONS          8
typedef struct STAR_TYP
        {
        int x,y;                   // position of star
        UCHAR color;               // color of star
        int plane;                 // plane that star is in

        } STAR, *STAR_PTR;

// Host provided functions
    extern void HostMaskKeys(void);
    extern void HostUnmaskKeys(void);
    extern long int HostGetConsoleInput(void);
    extern void GamePlaySounds(void);
    extern void HostDrawObject(int bitmapNumber,
    short int x,
    short int y);
    extern void HostDrawGameMenu(int left,
        int top,
        int width,
        int height);
    extern void HostDrawPoint(int x,
        int y);
    extern void HostFlushPoints(void);
    extern void HostNewScreenBuffer(short int buffer_width,
    short int buffer_height);
    extern void HostDrawBuffer(short int destX,
    short int destY,
    short int sourceX,
    short int sourceY,
    short int sourceWidth,
    short int sourceHeight);
    extern long int HostPeriodDuration(void);
// extern void HostSetForeColor(ColorType color);

// extern void HostSetBackColor(ColorType color);

    extern void HostOpenScreenBuffer(void);
    extern void HostCloseScreenBuffer(void);
    extern int HostGetBitmapWidth(int bitmapNumber);
    extern int HostGetBitmapHeight(int bitmapNumber);
	
	extern void HostPlayRumble(short int effect);
	extern TwGfxSurfaceType* LoadBitmap(Int16 aResID);
	extern void HostDrawBob(TwGfxPointType destPoint,TwGfxSurfaceType *curr_frame,TwGfxRectType source_rect);
	extern void HostDrawTransparentBob(TwGfxPointType destPoint,TwGfxSurfaceType *curr_frame,TwGfxRectType source_rect,TwGfxPackedRGBType Transparent_Colour);
	extern PARTICLE particles[MAX_PARTICLES]; // the particles for the particle engine
	extern HOSTPARTICLE hostparticles[MAX_PARTICLES];  // The draw system for particles
	extern HOSTPARTICLE_COLOUR hostparticles_colour[MAX_PARTICLES];
	extern float cos_look[361]; // 1 extra element so we can store 0-360 inclusive
	extern float sin_look[361];
	extern float particle_wind;    // assume it operates in the X direction
	extern float particle_gravity; // assume it operates in the Y direction
	extern void HostDrawParticle(int x, int y, int colour);
	extern TwGfxPackedRGBType RGBColour_range[24];
	extern void HostDrawParticles(int count);
	extern void HostDrawPixel(int x, int y, int colour);
	extern void HostDrawPixel16(int x, int y, TwGfxPackedRGBType RGBcolour);
	extern void Start_Mine(int override, int x, int y, int ai_state);
	extern void Start_Burst(int x, int y, int width, int height, int xv,int yv);
	extern void Start_Rock(int x, int y, int size,int xv, int yv);
	extern void Fire_Plasma(int x,int y, int xv, int yv, int source);
	extern void Draw_Scanner(void);
	extern void Init_Stations(void);
	extern void Reset_Stations(void);
	extern void Start_Station(int override, int x, int y);
	extern void Move_Stations(void);
	extern void Draw_Stations(void);
	extern void Delete_Stations(void);
	extern void Init_Mines(void);
	extern void Reset_Mines(void);
	extern void Move_Mines(void);
	extern void Draw_Mines(void);
	extern void Delete_Mines(void);
	extern void Init_Gunships(void);
	extern void Reset_Gunships(void);
	extern void Start_Gunship(int override, int x, int y, int dir);
	extern void Move_Gunships(void);
	extern void Draw_Gunships(void);
	extern void Delete_Gunships(void);
	extern void Init_Powerups(void);
	extern void Reset_Powerups(void);
	extern void Start_Powerup(int type, int x, int y);
	extern void Move_Powerups(void);
	extern void Draw_Powerups(void);
	extern void Delete_Powerups(void);
	extern void Init_Plasma(void);
	extern void Reset_Plasma(void);
	extern void Delete_Plasma(void);
	extern void Move_Plasma(void);
	extern void Draw_Plasma(void);
	extern void Init_Rocks(void);
	extern void Reset_Rocks(void);
	extern void Delete_Rocks(void);
	extern void Seed_Rocks(void);
	extern void Move_Rocks(void);
	extern void Start_Rock(int x, int y, int size,int xv, int yv);
	extern void Draw_Rocks(void);
	extern void Init_Bursts(void);
	extern void Reset_Bursts(void);
	extern void Delete_Bursts(void);
	extern void Move_Bursts(void);
	extern void Draw_Bursts(void);
	extern void Draw_Info(void);
	extern void Do_Intro(void);
	extern int Load_Player(void);
	extern void Create_Tables(void);
	extern float Fast_Distance_2D(float x, float y);
	extern void Game_Reset(void);
	extern int Game_Init();
	extern int Game_Shutdown();
	extern void HostDrawScaledBob(TwGfxRectType destRect,TwGfxSurfaceType *curr_frame,TwGfxRectType source_rect, BOB_PTR);
	extern void HostDrawBackground();
	extern void HostFontDrawChars(char *TextIn, int textLength, int x, int y);
	extern void PlayToneOS5(int EffectID);
	extern UInt8 CheckMidiDuration();
	extern void PlayMidiMusic(UInt16 MusicID);
	extern void stopMusic();
	extern void smfCallback(UInt32 aUserData);
	extern void startMusic(bool aRestart);
	extern bool CheckMusicPlaying();


	extern BOB wraith;                   // the player 
	extern BOB plasma[MAX_PLASMA];       // plasma pulses
	extern BOB rocks[MAX_ROCKS];         // the asteroids
	extern BOB rock_s, rock_m, rock_l;   // the master templates for surface info
	extern BOB bursts[MAX_BURSTS];       // the explosion bursts
	extern BOB gunships[MAX_GUNSHIPS];   // the gunships
	extern BOB powerups[MAX_POWERUPS];   // the powerups
	extern BOB stations[MAX_STATIONS];   // the starbase stations
	extern BOB mines[MAX_MINES];         // the predator mines
	extern BOB hud;                      // the art for the scanner hud
	extern BOB stationsmall;             // small station bob

	extern int rock_sizes[3]; // X,Y dims for scaler

	extern STAR stars[MAX_STARS];          // the star field


// player state variables
	extern int player_state;
	extern int player_score;   // the score
	extern int player_ships;   // ships left
	extern int player_regen_counter; // used to track when to regen
	extern int player_damage;   // damage of player
	extern int player_ammo;// ammo of player
	extern int player_counter;   // used for state transition tracking
	extern int player_regen_count;   // used to regenerate player
	extern int player_shield_count;  // used to display shields
	extern int ready_counter,        // used to draw a little "get ready"
     ready_state;
	extern int ammo_regen_counter;
	extern float mine_tracking_rate; // rate that mines track player

	extern int win_counter,          // tracks if player won
    player_won;   

	extern int station_id,          // uplink station id
    num_stations_destroyed;

	extern int intro_done;           // flags if intro has played already

	extern int game_paused,
    pause_debounce   ,
    huds_on          ,
    scanner_on       ,
    huds_debounce    ,
    scanner_debounce ;


// positional and state info for player
	extern float player_x,
      player_y,
	  player_dx, 
      player_dy, 
	  player_xv, 
	  player_yv,
      vel;

// these contain the virual cos, sin lookup tables for the 16 sector circle
	extern float cos_look16[16],
      sin_look16[16];

// sound id's
	extern int intro_music_id  ,
    main_music_id   ,
    ready_id        ,
    engines_id      ,
    scream_id       ,
    game_over_id    ,
    mine_powerup_id ,
    deactivate_id   ,
    main_menu_id    ,
    empty_id        ,
    beep0_id             ,
    beep1_id             ,
    station_blow_id      ,
    station_throb_id     ,
    ammopu_id            ,
    shldpu_id            ;

	extern int expl_ids[MAX_EXPL_SOUNDS];
	extern int fire_ids[MAX_FIRE_SOUNDS];
	extern int game_state;   // initial game state
// Game provided functions.
#define game_pause 1
#define game_run 0

    extern void game_set_pause(int value);
    extern int game_get_pause(void);
    extern void game_new(int width,
        int height,
        int random_number_seed);
	extern int game_exists(void);
	extern int game_advance(unsigned long int keys, long int joystickX, long int joystickY, 
			 long int ActionA, long int ActionB, long int ActionC, long int ActionD, 
			 long int TriggerA, long int TriggerB);

    extern void game_delete(void);


#ifdef __cplusplus
}
#endif

