/*
	File:			random.c

	Description:	Use our own random number generator for performance reasons.  It's statically linked so it is
					quickly accessed.  We pick a quick but useful enough method for number generation.  The method 
					produces the same string when given the same seed, which is important for repeatability for testing.

	Author:		

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/


#include "host.h"
#include "random.h"



static uint32 RandomNumberSeed;


/***********************************************************************
 *
 * FUNCTION:     RandomNumberGetSeed
 *
 * DESCRIPTION:  Return the seed.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     the seed
 *
 ***********************************************************************/

uint32
random_number_get_seed(void)
{
    return RandomNumberSeed;
}


/***********************************************************************
 *
 * FUNCTION:     RandomNumberSetSeed
 *
 * DESCRIPTION:  Set the seed.
 *
 * PARAMETERS:   number - number to use as the seed.
 *
 * RETURNED:     nothing
 *
 ***********************************************************************/

void
random_number_set_seed(uint32 number)
{
    RandomNumberSeed = number;
}


/***********************************************************************
 *
 * FUNCTION:     RandomNumber
 *
 * DESCRIPTION:  Return a number from 0 to range - 1.
 *
 * PARAMETERS:   range - range to pick a number from.
 *
 * RETURNED:     a number from 0 to range - 1
 *
 ***********************************************************************/
#define	randomMax		0xFFFF  // Max value returned

uint16
random_number(int16 range)
{
    uint32 result;

//   assert(range >= 0);

    // Generate the new seed
    RandomNumberSeed *= 2147001325;
    RandomNumberSeed += 715136305;

    // Return the random number
    result = RandomNumberSeed >> 16;

    // Make the result in proportion to the range requested.
    result = (result * range) / ((uint32) randomMax + 1);

    return (uint16) result;
}
int range(int n) {
   unsigned long div;
   int r;

   div = 4294967295/n;

   do {
      r = rand() / div;
   } while (r >= n);

   return r;
}
/* -------------------------------------------------------------- 

   range2

   returns an integer between low and high inclusive

*/


int range2(int low, int high) {
   return low + range(high-low+1);
}
/* ----------------------------------------------------------------------*/
/* Combination of 3 tausworthe generators -- assumes 32-bit integers */
/* Source: Pierre L'Ecuyer, "Maximally Equidistributed Combined Tausworthe
   Generators". Mathematics of Computation, vol.65, no.213(1996), pp203--213.*/
/* Special thanks to Joushua Grass (of Microsoft) for pointing out an error
   in the initialization routine "rand_seed()" */

UInt32 rand();             /* returns a random 32 bit integer */
void rand_seed( UInt32, UInt32, UInt32 );

/* return a random float >= 0 and < 1 */
#define rand_float          ((double)rand() / 4294967296.0)

static unsigned int s1=390451501, s2=613566701, s3=858993401;  /* The seeds */
static unsigned mask1, mask2, mask3;
static int shft1, shft2, shft3, k1=31, k2=29, k3=28;

/* use either of the following two sets of parameters*/
static int q1=13, q2=2, q3=3, p1=12, p2=4, p3=17;
/* static int q1=3, q2=2, q3=13, p1=20, p2=16, p3=7; */


UInt32 rand()
   {
   UInt32 b;
  
   b  = ((s1 << q1)^s1) >> shft1;
   s1 = ((s1 & mask1) << p1) ^ b;
   b  = ((s2 << q2) ^ s2) >> shft2;
   s2 = ((s2 & mask2) << p2) ^ b;
   b  = ((s3 << q3) ^ s3) >> shft3;
   s3 = ((s3 & mask3) << p3) ^ b;
   return (s1 ^ s2 ^ s3);
   }


void rand_seed( UInt32 a, UInt32 b, UInt32 c )
   {
   static UInt32 x = 4294967295U;

   shft1 = k1-p1;
   shft2 = k2-p2;
   shft3 = k3-p3;
   mask1 = x << (32-k1);
   mask2 = x << (32-k2);
   mask3 = x << (32-k3);
   if (a > (1 << (32-k1))) s1 = a;
   if (b > (1 << (32-k2))) s2 = b;
   if (c > (1 << (32-k3))) s3 = c;
   rand();
   }
